<?php

/**

 * CodeIgniter

 *

 * An open source application development framework for PHP

 *

 * This content is released under the MIT License (MIT)

 *

 * Copyright (c) 2014 - 2017, British Columbia Institute of Technology

 *

 * Permission is hereby granted, free of charge, to any person obtaining a copy

 * of this software and associated documentation files (the "Software"), to deal

 * in the Software without restriction, including without limitation the rights

 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell

 * copies of the Software, and to permit persons to whom the Software is

 * furnished to do so, subject to the following conditions:

 *

 * The above copyright notice and this permission notice shall be included in

 * all copies or substantial portions of the Software.

 *

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR

 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,

 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE

 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER

 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,

 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN

 * THE SOFTWARE.

 *

 * @package	CodeIgniter

 * @author	EllisLab Dev Team

 * @copyright	Copyright (c) 2008 - 2014, EllisLab, Inc. (https://ellislab.com/)

 * @copyright	Copyright (c) 2014 - 2017, British Columbia Institute of Technology (http://bcit.ca/)

 * @license	http://opensource.org/licenses/MIT	MIT License

 * @link	https://codeigniter.com

 * @since	Version 1.4.1

 * @filesource

 */

defined('BASEPATH') OR exit('No direct script access allowed');



/**

 * Oracle Forge Class

 *

 * @category	Database

 * @author		EllisLab Dev Team

 * @link		https://codeigniter.com/user_guide/database/

 */

class CI_DB_oci8_forge extends CI_DB_forge {



	/**

	 * CREATE DATABASE statement

	 *

	 * @var	string

	 */

	protected $_create_database	= FALSE;



	/**

	 * CREATE TABLE IF statement

	 *

	 * @var	string

	 */

	protected $_create_table_if	= FALSE;



	/**

	 * DROP DATABASE statement

	 *

	 * @var	string

	 */

	protected $_drop_database	= FALSE;



	/**

	 * DROP TABLE IF statement

	 *

	 * @var	string

	 */

	protected $_drop_table_if	= FALSE;



	/**

	 * UNSIGNED support

	 *

	 * @var	bool|array

	 */

	protected $_unsigned		= FALSE;



	// --------------------------------------------------------------------



	/**

	 * ALTER TABLE

	 *

	 * @param	string	$alter_type	ALTER type

	 * @param	string	$table		Table name

	 * @param	mixed	$field		Column definition

	 * @return	string|string[]

	 */

	protected function _alter_table($alter_type, $table, $field)

	{

		if ($alter_type === 'DROP')

		{

			return parent::_alter_table($alter_type, $table, $field);

		}

		elseif ($alter_type === 'CHANGE')

		{

			$alter_type = 'MODIFY';

		}



		$sql = 'ALTER TABLE '.$this->db->escape_identifiers($table);

		$sqls = array();

		for ($i = 0, $c = count($field); $i < $c; $i++)

		{

			if ($field[$i]['_literal'] !== FALSE)

			{

				$field[$i] = "\n\t".$field[$i]['_literal'];

			}

			else

			{

				$field[$i]['_literal'] = "\n\t".$this->_process_column($field[$i]);



				if ( ! empty($field[$i]['comment']))

				{

					$sqls[] = 'COMMENT ON COLUMN '

						.$this->db->escape_identifiers($table).'.'.$this->db->escape_identifiers($field[$i]['name'])

						.' IS '.$field[$i]['comment'];

				}



				if ($alter_type === 'MODIFY' && ! empty($field[$i]['new_name']))

				{

					$sqls[] = $sql.' RENAME COLUMN '.$this->db->escape_identifiers($field[$i]['name'])

						.' TO '.$this->db->escape_identifiers($field[$i]['new_name']);

				}



				$field[$i] = "\n\t".$field[$i]['_literal'];

			}

		}



		$sql .= ' '.$alter_type.' ';

		$sql .= (count($field) === 1)

				? $field[0]

				: '('.implode(',', $field).')';



		// RENAME COLUMN must be executed after MODIFY

		array_unshift($sqls, $sql);

		return $sqls;

	}



	// --------------------------------------------------------------------



	/**

	 * Field attribute AUTO_INCREMENT

	 *

	 * @param	array	&$attributes

	 * @param	array	&$field

	 * @return	void

	 */

	protected function _attr_auto_increment(&$attributes, &$field)

	{

		// Not supported - sequences and triggers must be used instead

	}



	// --------------------------------------------------------------------



	/**

	 * Field attribute TYPE

	 *

	 * Performs a data type mapping between different databases.

	 *

	 * @param	array	&$attributes

	 * @return	void

	 */

	protected function _attr_type(&$attributes)

	{

		switch (strtoupper($attributes['TYPE']))

		{

			case 'TINYINT':

				$attributes['TYPE'] = 'NUMBER';

				return;

			case 'MEDIUMINT':

				$attributes['TYPE'] = 'NUMBER';

				return;

			case 'INT':

				$attributes['TYPE'] = 'NUMBER';

				return;

			case 'BIGINT':

				$attributes['TYPE'] = 'NUMBER';

				return;

			default: return;

		}

	}

}

